@@ -8,6 +8,7 @@ import string  # pylint: disable=deprecated-module
 from inspect import Parameter
 from typing import Dict, List
 
+import docstring_to_markdown
 import jedi.api.errors
 import jedi.inference.references
 import jedi.settings
@@ -28,7 +29,6 @@ from pygls.types import (
     SymbolInformation,
     SymbolKind,
 )
-from pygls.uris import from_fs_path
 from pygls.workspace import Document
 
 from .initialize_params_parser import InitializeParamsParser
@@ -46,6 +46,10 @@ def set_jedi_settings(  # pylint: disable=invalid-name
         )
     )
 
+    jedi.settings.case_insensitive_completion = (
+        ip.initializationOptions_jediSettings_caseInsensitiveCompletion
+    )
+
 
 def script(project: Project, document: Document) -> Script:
     """Simplifies getting jedi Script."""
@@ -70,7 +74,7 @@ def lsp_range(name: Name) -> Range:
 
 def lsp_location(name: Name) -> Location:
     """Get LSP location from Jedi definition."""
-    return Location(uri=from_fs_path(name.module_path), range=lsp_range(name))
+    return Location(uri=name.module_path.as_uri(), range=lsp_range(name))
 
 
 def lsp_symbol_information(name: Name) -> SymbolInformation:
@@ -123,6 +127,7 @@ def lsp_document_symbols(names: List[Name]) -> List[DocumentSymbol]:
             kind=get_lsp_symbol_type(name.type),
             range=_document_symbol_range(name),
             selection_range=lsp_range(name),
+            detail=name.description,
             children=[],
         )
         parent = name.parent()
@@ -139,7 +144,9 @@ def lsp_document_symbols(names: List[Name]) -> List[DocumentSymbol]:
         ):
             # special case for __init__ method in class; names defined here
             symbol.kind = SymbolKind.Method
-            _name_lookup[parent].children.append(symbol)  # type: ignore
+            parent_symbol = _name_lookup[parent]
+            assert parent_symbol.children is not None
+            parent_symbol.children.append(symbol)
             _name_lookup[name] = symbol
         elif parent not in _name_lookup:
             # unqualified names are not included in the tree
@@ -149,7 +156,9 @@ def lsp_document_symbols(names: List[Name]) -> List[DocumentSymbol]:
         ):
             # handle attribute creation on __init__ method
             symbol.kind = SymbolKind.Property
-            _name_lookup[parent].children.append(symbol)  # type: ignore
+            parent_symbol = _name_lookup[parent]
+            assert parent_symbol.children is not None
+            parent_symbol.children.append(symbol)
         elif parent.type == "class":
             # children are added for class scopes
             if name.type == "function":
@@ -159,7 +168,9 @@ def lsp_document_symbols(names: List[Name]) -> List[DocumentSymbol]:
                 symbol.kind = SymbolKind.Method
             else:
                 symbol.kind = SymbolKind.Property
-            _name_lookup[parent].children.append(symbol)  # type: ignore
+            parent_symbol = _name_lookup[parent]
+            assert parent_symbol.children is not None
+            parent_symbol.children.append(symbol)
     return results
 
 
@@ -206,7 +217,7 @@ def line_column(jedi_script: Script, position: Position) -> Dict[str, int]:
     line_length = len(lines[position.line])
     return dict(
         line=position.line + 1,
-        column=min(position.character, line_length - 1),
+        column=min(position.character, line_length - 1 if line_length else 0),
     )
 
 
@@ -235,7 +246,8 @@ def compare_names(name1: Name, name2: Name) -> bool:
     without needing to directly import anything from jedi into
     `server.py`
     """
-    return name1 == name2
+    equal: bool = name1 == name2
+    return equal
 
 
 def complete_sort_name(name: Completion) -> str:
@@ -290,11 +302,6 @@ def get_snippet_signature(signature: Signature) -> str:
             signature_list.append(result)
             count += 1
             continue
-        if param.kind == Parameter.KEYWORD_ONLY:
-            result = param_name + "=${" + f"{count}:..." + "}"
-            signature_list.append(result)
-            count += 1
-            continue
     if not signature_list:
         return "($0)"
     return "(" + ", ".join(signature_list) + ")$0"
@@ -317,7 +324,8 @@ def is_import(script_: Script, line: int, column: int) -> bool:
     name = script_._get_module_context().create_name(tree_name)
     if name is None:
         return False
-    return name.is_import()
+    name_is_import: bool = name.is_import()
+    return name_is_import
 
 
 _LSP_TYPE_FOR_SNIPPET = {
@@ -325,33 +333,46 @@ _LSP_TYPE_FOR_SNIPPET = {
     CompletionItemKind.Function,
 }
 
+_MOST_RECENT_COMPLETIONS: Dict[str, Completion] = {}
+
+
+def clear_completions_cache() -> None:
+    """Clears the cache of completions used for completionItem/resolve."""
+    _MOST_RECENT_COMPLETIONS.clear()
+
 
 def lsp_completion_item(
-    name: Completion,
+    completion: Completion,
     char_before_cursor: str,
     enable_snippets: bool,
+    resolve_eagerly: bool,
     markup_kind: MarkupKind,
 ) -> CompletionItem:
     """Using a Jedi completion, obtain a jedi completion item."""
-    name_name = name.name
-    name_clean = clean_completion_name(name_name, char_before_cursor)
-    lsp_type = get_lsp_completion_type(name.type)
+    completion_name = completion.name
+    name_clean = clean_completion_name(completion_name, char_before_cursor)
+    lsp_type = get_lsp_completion_type(completion.type)
     completion_item = CompletionItem(
-        label=name_name,
-        filter_text=name_name,
+        label=completion_name,
+        filter_text=completion_name,
         kind=lsp_type,
-        detail=name.description,
-        documentation=MarkupContent(kind=markup_kind, value=name.docstring()),
-        sort_text=complete_sort_name(name),
+        sort_text=complete_sort_name(completion),
         insert_text=name_clean,
         insert_text_format=InsertTextFormat.PlainText,
     )
+
+    _MOST_RECENT_COMPLETIONS[completion_name] = completion
+    if resolve_eagerly:
+        completion_item = lsp_completion_item_resolve(
+            completion_item, markup_kind=markup_kind
+        )
+
     if not enable_snippets:
         return completion_item
     if lsp_type not in _LSP_TYPE_FOR_SNIPPET:
         return completion_item
 
-    signatures = name.get_signatures()
+    signatures = completion.get_signatures()
     if not signatures:
         return completion_item
 
@@ -359,12 +380,56 @@ def lsp_completion_item(
         snippet_signature = get_snippet_signature(signatures[0])
     except Exception:  # pylint: disable=broad-except
         return completion_item
-    new_text = name_name + snippet_signature
+    new_text = completion_name + snippet_signature
     completion_item.insertText = new_text
     completion_item.insertTextFormat = InsertTextFormat.Snippet
     return completion_item
 
 
+def convert_docstring(docstring: str, markup_kind: MarkupKind) -> str:
+    """Take a docstring and convert it to markup kind if possible.
+
+    Currently only supports markdown conversion; MarkupKind can only be
+    plaintext or markdown as of LSP 3.16.
+
+    NOTE: Since docstring_to_markdown is a new library, I add broad exception
+    handling in case docstring_to_markdown.convert produces unexpected
+    behavior.
+    """
+    if markup_kind == MarkupKind.Markdown:
+        try:
+            return docstring_to_markdown.convert(docstring)
+        except docstring_to_markdown.UnknownFormatError:
+            return (
+                "```\n" + str(docstring) + "\n```\n"
+                if docstring
+                else docstring
+            )
+        except Exception as error:  # pylint: disable=broad-except
+            return (
+                docstring
+                + "\n"
+                + "jedi-language-server error: "
+                + "Uncaught exception while converting docstring to markdown. "
+                + "Please open issue at "
+                + "https://github.com/pappasam/jedi-language-server/issues. "
+                + f"Traceback:\n{error}"
+            )
+    return docstring
+
+
+def lsp_completion_item_resolve(
+    item: CompletionItem,
+    markup_kind: MarkupKind,
+) -> CompletionItem:
+    """Resolve completion item using cached jedi completion data."""
+    completion = _MOST_RECENT_COMPLETIONS[item.label]
+    item.detail = completion.description
+    docstring = convert_docstring(completion.docstring(), markup_kind)
+    item.documentation = MarkupContent(kind=markup_kind, value=docstring)
+    return item
+
+
 def random_var(
     beginning: str,
     random_length: int = 8,
