@@ -1,4 +1,4 @@
-"""Jedi Language Server
+"""Jedi Language Server.
 
 Creates the language server constant and wraps "features" with it.
 
@@ -7,13 +7,14 @@ Official language server spec:
 """
 
 import itertools
-from typing import List, Optional, Union
+from typing import Any, List, Optional, Union
 
 from jedi import Project
 from jedi.api.refactoring import RefactoringError
 from pygls.features import (
     CODE_ACTION,
     COMPLETION,
+    COMPLETION_ITEM_RESOLVE,
     DEFINITION,
     DOCUMENT_HIGHLIGHT,
     DOCUMENT_SYMBOL,
@@ -24,6 +25,7 @@ from pygls.features import (
     TEXT_DOCUMENT_DID_CHANGE,
     TEXT_DOCUMENT_DID_OPEN,
     TEXT_DOCUMENT_DID_SAVE,
+    WORKSPACE_DID_CHANGE_CONFIGURATION,
     WORKSPACE_SYMBOL,
 )
 from pygls.protocol import LanguageServerProtocol
@@ -32,8 +34,10 @@ from pygls.types import (
     CodeAction,
     CodeActionKind,
     CodeActionParams,
+    CompletionItem,
     CompletionList,
     CompletionParams,
+    DidChangeConfigurationParams,
     DidChangeTextDocumentParams,
     DidOpenTextDocumentParams,
     DidSaveTextDocumentParams,
@@ -45,6 +49,7 @@ from pygls.types import (
     InitializeResult,
     Location,
     MarkupContent,
+    MarkupKind,
     RenameParams,
     SymbolInformation,
     TextDocumentPositionParams,
@@ -53,7 +58,6 @@ from pygls.types import (
 )
 
 from . import jedi_utils, pygls_utils, text_edit_utils
-from .constants import JEDI_IGNORE_FOLDERS
 from .initialize_params_parser import InitializeParamsParser
 from .pygls_type_overrides import (
     ParameterInformation,
@@ -84,7 +88,7 @@ class JediLanguageServerProtocol(LanguageServerProtocol):
                 server.feature(TEXT_DOCUMENT_DID_CHANGE)(did_change)
             if ip.initializationOptions_diagnostics_didSave:
                 server.feature(TEXT_DOCUMENT_DID_SAVE)(did_save)
-        initialize_result = super().bf_initialize(params)
+        initialize_result: InitializeResult = super().bf_initialize(params)
         server.project = Project(
             path=server.workspace.root_path,
             added_sys_path=ip.initializationOptions_workspace_extraPaths,
@@ -104,7 +108,7 @@ class JediLanguageServer(LanguageServer):
 
     project: Project
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         self.initialize_params = InitializeParamsParser()
         super().__init__(*args, **kwargs)
 
@@ -115,6 +119,36 @@ SERVER = JediLanguageServer(protocol_cls=JediLanguageServerProtocol)
 # Server capabilities
 
 
+@SERVER.feature(COMPLETION_ITEM_RESOLVE)
+def completion_item_resolve(
+    server: JediLanguageServer, params: Any
+) -> CompletionItem:
+    """Resolves documentation and detail of given completion item."""
+    markup_kind = _choose_markup(server)
+    # note: params is not a CompletionItem
+    # but a namedtuple complying with CompletionItem protocol
+    item = CompletionItem(
+        label=params.label,
+        kind=getattr(params, "kind", None),
+        detail=getattr(params, "detail", None),
+        documentation=getattr(params, "documentation", None),
+        deprecated=getattr(params, "deprecated", None),
+        preselect=getattr(params, "preselect", None),
+        sort_text=getattr(params, "sortText", None),
+        filter_text=getattr(params, "filterText", None),
+        insert_text=getattr(params, "insertText", None),
+        insert_text_format=getattr(params, "insertTextFormat", None),
+        text_edit=getattr(params, "textEdit", None),
+        additional_text_edits=getattr(params, "additionalTextEdits", None),
+        commit_characters=getattr(params, "commitCharacters", None),
+        command=getattr(params, "command", None),
+        data=getattr(params, "data", None),
+    )
+    return jedi_utils.lsp_completion_item_resolve(
+        item, markup_kind=markup_kind
+    )
+
+
 @SERVER.feature(COMPLETION, trigger_characters=[".", "'", '"'])
 def completion(
     server: JediLanguageServer, params: CompletionParams
@@ -124,23 +158,16 @@ def completion(
     jedi_script = jedi_utils.script(server.project, document)
     jedi_lines = jedi_utils.line_column(jedi_script, params.position)
     completions_jedi = jedi_script.complete(**jedi_lines)
-    markup_preferred = (
-        server.initialize_params.initializationOptions_markupKindPreferred
-    )
-    markup_supported = (
-        server.initialize_params.capabilities_textDocument_completion_completionItem_documentationFormat
-    )
-    markup_kind = (
-        markup_preferred
-        if markup_preferred in markup_supported
-        else markup_supported[0]
-    )
     snippet_support = (
         server.initialize_params.capabilities_textDocument_completion_completionItem_snippetSupport
     )
     snippet_disable = (
         server.initialize_params.initializationOptions_completion_disableSnippets
     )
+    resolve_eagerly = (
+        server.initialize_params.initializationOptions_completion_resolveEagerly
+    )
+    markup_kind = _choose_markup(server)
     is_import_context = jedi_utils.is_import(
         script_=jedi_script,
         line=jedi_lines["line"],
@@ -153,11 +180,13 @@ def completion(
         document=server.workspace.get_document(params.textDocument.uri),
         position=params.position,
     )
+    jedi_utils.clear_completions_cache()
     completion_items = [
         jedi_utils.lsp_completion_item(
-            name=completion,
+            completion=completion,
             char_before_cursor=char_before_cursor,
             enable_snippets=enable_snippets,
+            resolve_eagerly=resolve_eagerly,
             markup_kind=markup_kind,
         )
         for completion in completions_jedi
@@ -255,21 +284,12 @@ def hover(
         docstring = name.docstring()
         if not docstring:
             continue
-        markup_preferred = (
-            server.initialize_params.initializationOptions_markupKindPreferred
-        )
-        markup_supported = (
-            server.initialize_params.capabilities_textDocument_hover_contentFormat
-        )
-        markup_kind = (
-            markup_preferred
-            if markup_preferred in markup_supported
-            else markup_supported[0]
-        )
-        contents = MarkupContent(kind=markup_kind, value=docstring)
+        markup_kind = _choose_markup(server)
+        docstring_clean = jedi_utils.convert_docstring(docstring, markup_kind)
+        contents = MarkupContent(kind=markup_kind, value=docstring_clean)
         document = server.workspace.get_document(params.textDocument.uri)
         _range = pygls_utils.current_word_range(document, params.position)
-        return Hover(contents=contents, range=_range)
+        return Hover(contents=contents, range=_range)  # type: ignore
     return None
 
 
@@ -324,12 +344,12 @@ def document_symbol(
     return symbol_information if symbol_information else None
 
 
-def _ignore_folder(path_check: str) -> bool:
+def _ignore_folder(path_check: str, jedi_ignore_folders: List[str]) -> bool:
     """Determines whether there's an ignore folder in the path.
 
     Intended to be used with the `workspace_symbol` function
     """
-    for ignore_folder in JEDI_IGNORE_FOLDERS:
+    for ignore_folder in jedi_ignore_folders:
         if f"/{ignore_folder}/" in path_check:
             return True
     return False
@@ -341,7 +361,8 @@ def workspace_symbol(
 ) -> Optional[List[SymbolInformation]]:
     """Document Python workspace symbols.
 
-    Returns up to 20 symbols, ignoring the following symbols:
+    Returns up to maxSymbols, or all symbols if maxSymbols is <= 0, ignoring
+    the following symbols:
 
     1. Those that don't have a module_path associated with them (built-ins)
     2. Those that are not rooted in the current workspace.
@@ -349,14 +370,24 @@ def workspace_symbol(
     """
     names = server.project.complete_search(params.query)
     workspace_root = server.workspace.root_path
+    ignore_folders = (
+        server.initialize_params.initializationOptions_workspace_symbols_ignoreFolders
+    )
     _symbols = (
         jedi_utils.lsp_symbol_information(name)
         for name in names
         if name.module_path
-        and name.module_path.startswith(workspace_root)
-        and not _ignore_folder(name.module_path)
+        and str(name.module_path).startswith(workspace_root)
+        and not _ignore_folder(str(name.module_path), ignore_folders)
+    )
+    max_symbols = (
+        server.initialize_params.initializationOptions_workspace_symbols_maxSymbols
+    )
+    symbols = (
+        list(itertools.islice(_symbols, max_symbols))
+        if max_symbols > 0
+        else list(_symbols)
     )
-    symbols = list(itertools.islice(_symbols, 20))
     return symbols if symbols else None
 
 
@@ -372,7 +403,9 @@ def rename(
         refactoring = jedi_script.rename(new_name=params.newName, **jedi_lines)
     except RefactoringError:
         return None
-    changes = text_edit_utils.lsp_document_changes(refactoring)
+    changes = text_edit_utils.lsp_document_changes(
+        server.workspace, refactoring
+    )
     return WorkspaceEdit(document_changes=changes) if changes else None  # type: ignore
 
 
@@ -404,11 +437,11 @@ def code_action(
             # refactor this at some point; control flow with exception == bad
             raise RefactoringError("inline only viable for single-line range")
         inline_refactoring = jedi_script.inline(**jedi_lines)
-    except RefactoringError:
-        inline_changes = []  # type: ignore
+    except (RefactoringError, AttributeError, IndexError):
+        inline_changes = []
     else:
         inline_changes = text_edit_utils.lsp_document_changes(
-            inline_refactoring
+            server.workspace, inline_refactoring
         )
     if inline_changes:
         code_actions.append(
@@ -426,11 +459,11 @@ def code_action(
         extract_variable_refactoring = jedi_script.extract_variable(
             new_name=extract_var, **jedi_lines_extract
         )
-    except RefactoringError:
-        extract_variable_changes = []  # type: ignore
+    except (RefactoringError, AttributeError, IndexError):
+        extract_variable_changes = []
     else:
         extract_variable_changes = text_edit_utils.lsp_document_changes(
-            extract_variable_refactoring
+            server.workspace, extract_variable_refactoring
         )
     if extract_variable_changes:
         code_actions.append(
@@ -448,11 +481,11 @@ def code_action(
         extract_function_refactoring = jedi_script.extract_function(
             new_name=extract_func, **jedi_lines_extract
         )
-    except RefactoringError:
-        extract_function_changes = []  # type: ignore
+    except (RefactoringError, AttributeError, IndexError):
+        extract_function_changes = []
     else:
         extract_function_changes = text_edit_utils.lsp_document_changes(
-            extract_function_refactoring
+            server.workspace, extract_function_refactoring
         )
     if extract_function_changes:
         code_actions.append(
@@ -468,10 +501,22 @@ def code_action(
     return code_actions if code_actions else None
 
 
+@SERVER.feature(WORKSPACE_DID_CHANGE_CONFIGURATION)
+def did_change_configuration(
+    server: JediLanguageServer,  # pylint: disable=unused-argument
+    params: DidChangeConfigurationParams,  # pylint: disable=unused-argument
+) -> None:
+    """Implement event for workspace/didChangeConfiguration.
+
+    Currently does nothing, but necessary for pygls. See::
+        <https://github.com/pappasam/jedi-language-server/issues/58>
+    """
+
+
 # Static capability or initializeOptions functions that rely on a specific
 # client capability or user configuration. These are associated with
 # JediLanguageServer within JediLanguageServerProtocol.bf_initialize
-def _publish_diagnostics(server: JediLanguageServer, uri: str):
+def _publish_diagnostics(server: JediLanguageServer, uri: str) -> None:
     """Helper function to publish diagnostics for a file."""
     document = server.workspace.get_document(uri)
     jedi_script = jedi_utils.script(server.project, document)
@@ -481,7 +526,9 @@ def _publish_diagnostics(server: JediLanguageServer, uri: str):
 
 
 # TEXT_DOCUMENT_DID_SAVE
-def did_save(server: JediLanguageServer, params: DidSaveTextDocumentParams):
+def did_save(
+    server: JediLanguageServer, params: DidSaveTextDocumentParams
+) -> None:
     """Actions run on textDocument/didSave."""
     _publish_diagnostics(server, params.textDocument.uri)
 
@@ -489,12 +536,29 @@ def did_save(server: JediLanguageServer, params: DidSaveTextDocumentParams):
 # TEXT_DOCUMENT_DID_CHANGE
 def did_change(
     server: JediLanguageServer, params: DidChangeTextDocumentParams
-):
+) -> None:
     """Actions run on textDocument/didChange."""
     _publish_diagnostics(server, params.textDocument.uri)
 
 
 # TEXT_DOCUMENT_DID_OPEN
-def did_open(server: JediLanguageServer, params: DidOpenTextDocumentParams):
+def did_open(
+    server: JediLanguageServer, params: DidOpenTextDocumentParams
+) -> None:
     """Actions run on textDocument/didOpen."""
     _publish_diagnostics(server, params.textDocument.uri)
+
+
+def _choose_markup(server: JediLanguageServer) -> MarkupKind:
+    """Returns the preferred or first of supported markup kinds."""
+    markup_preferred = (
+        server.initialize_params.initializationOptions_markupKindPreferred
+    )
+    markup_supported = (
+        server.initialize_params.capabilities_textDocument_completion_completionItem_documentationFormat
+    )
+    return MarkupKind(
+        markup_preferred
+        if markup_preferred in markup_supported
+        else markup_supported[0]
+    )
