@@ -6,7 +6,7 @@ This module is a bridge between `jedi.Refactoring` and
 
 
 import difflib
-from typing import Dict, Iterator, List, NamedTuple, Union
+from typing import Dict, Iterator, List, NamedTuple, Optional, Union
 
 from jedi.api.refactoring import ChangedFile, Refactoring
 from pygls.types import (
@@ -17,14 +17,17 @@ from pygls.types import (
     TextEdit,
     VersionedTextDocumentIdentifier,
 )
-from pygls.uris import from_fs_path
+from pygls.workspace import Workspace
 
 
 class RenameFile:  # pylint: disable=too-few-public-methods
     """Pygls has bug right now, this would be simple pull request."""
 
     def __init__(
-        self, old_uri: str, new_uri: str, options: RenameFileOptions = None
+        self,
+        old_uri: str,
+        new_uri: str,
+        options: Optional[RenameFileOptions] = None,
     ):
         # pylint: disable=invalid-name
         self.kind = "rename"
@@ -34,13 +37,14 @@ class RenameFile:  # pylint: disable=too-few-public-methods
 
 
 def lsp_document_changes(
+    workspace: Workspace,
     refactoring: Refactoring,
 ) -> List[Union[TextDocumentEdit, RenameFile]]:
     """Get lsp text document edits from Jedi refactoring.
 
     This is the main public function that you probably want
     """
-    converter = RefactoringConverter(refactoring)
+    converter = RefactoringConverter(workspace, refactoring)
     return [
         *converter.lsp_text_document_edits(),
         *converter.lsp_renames(),
@@ -50,15 +54,16 @@ def lsp_document_changes(
 class RefactoringConverter:
     """Convert jedi Refactoring objects into renaming machines."""
 
-    def __init__(self, refactoring: Refactoring) -> None:
+    def __init__(self, workspace: Workspace, refactoring: Refactoring) -> None:
+        self.workspace = workspace
         self.refactoring = refactoring
 
     def lsp_renames(self) -> Iterator[RenameFile]:
         """Get all File rename operations."""
         for old_name, new_name in self.refactoring.get_renames():
             yield RenameFile(
-                old_uri=from_fs_path(old_name),
-                new_uri=from_fs_path(new_name),
+                old_uri=old_name.as_uri(),
+                new_uri=new_name.as_uri(),
                 options=RenameFileOptions(
                     ignore_if_exists=True, overwrite=True
                 ),
@@ -68,12 +73,14 @@ class RefactoringConverter:
         """Get all text document edits."""
         changed_files = self.refactoring.get_changed_files()
         for path, changed_file in changed_files.items():
-            uri = from_fs_path(path)
+            uri = path.as_uri()
+            document = self.workspace.get_document(uri)
+            version = 0 if document.version is None else document.version
             text_edits = lsp_text_edits(changed_file)
             yield TextDocumentEdit(
                 text_document=VersionedTextDocumentIdentifier(
                     uri=uri,
-                    version=None,  # type: ignore
+                    version=version,
                 ),
                 edits=text_edits,
             )
@@ -96,7 +103,7 @@ def lsp_text_edits(changed_file: ChangedFile) -> List[TextEdit]:
     for opcode in get_opcodes(old_code, new_code):
         if opcode.op in _OPCODES_CHANGE:
             start = opcode_position_lookup_old[opcode.old_start]
-            end = opcode_position_lookup_old[opcode.old_end]
+            end = opcode_position_lookup_old[opcode.old_end - 1]
             start_char = opcode.old_start - start.range_start
             end_char = opcode.old_end - end.range_start
             new_text = new_code[opcode.new_start : opcode.new_end]
@@ -176,7 +183,7 @@ def get_opcode_position_lookup(
     start = 0
     for line, code_line in enumerate(original_lines):
         end = start + len(code_line)
-        key = range(start, end + 1)  # must be 1 greater than last item
+        key = range(start, end)
         line_lookup[key] = LinePosition(start, end, line, code_line)
         start = end
     return line_lookup
